package com.sybase.sup.samples.objectapi;

import com.sybase.collections.GenericList;
import com.sybase.mobile.Application;
import com.sybase.mobile.ConnectionProperties;
import com.sybase.mobile.RegistrationStatus;
import com.sybase.persistence.DefaultCallbackHandler;
import com.sybase.persistence.LoginCredentials;
import com.sybase.persistence.SynchronizationAction;
import com.sybase.persistence.SynchronizationContext;
import com.sybase.persistence.SynchronizationGroup;
import com.sybase.persistence.SynchronizationStatus;

import SUP101.SUP101DB;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;

import android.view.*;
import android.widget.*;
import android.widget.AdapterView.OnItemClickListener;

public class SUP101SampleActivity extends Activity
{
    private static final int REQUEST_DETAIL = 99;
    private static String USERNAME = "supAdmin";
    private static String PASSWORD = "s3pAdmin";
    private static String HOST = "10.0.2.2";
    private static int PORT = 5001;
    private static int TIMEOUT = 600;
    private CustomerListAdapter adapter;

    private static volatile boolean initializationDone = false;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        initializeApplication();
    }

    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
        if (requestCode == REQUEST_DETAIL)
        {
            if (resultCode == RESULT_OK)
            {
                SUP101SampleActivity.this.adapter.refreshUI(true);
            }
        }
    }

    private void initializeApplication()
    {       
        final ProgressDialog dialog = new ProgressDialog(this);

        dialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        dialog.setTitle("on boarding ...");
        dialog.setMessage("Please wait while download initial data...");
        dialog.setIndeterminate(false);
        dialog.setProgress(100);
        dialog.setCancelable(true);
        dialog.show();

        new Thread()
        {
            @Override
            public void run()
            {
                try
                {
                    int count = 0;
                    while (!initializationDone)
                    {
                        dialog.setProgress(++count);
                        Thread.sleep(100);

                        if (count == 100)
                        {
                            count = 0;
                        }
                    }
                    dialog.cancel();
                }
                catch (Exception e)
                {
                    dialog.cancel();
                }
            }
        }.start();

        Application app = Application.getInstance();
        app.setApplicationIdentifier("SUP101");
        app.setApplicationContext(SUP101SampleActivity.this);

        new Thread(new Runnable()
        {
            @Override
            public void run()
            {
                try
                {
                    Application app = Application.getInstance();
                    SUP101DB.registerCallbackHandler(new CustomerDBCallback());
                    SUP101DB.setApplication(app);
                    SUP101DB.getSynchronizationProfile().setServerName(HOST); // Convenience only

                    ConnectionProperties connProps = app.getConnectionProperties();
                    LoginCredentials loginCredentials = new LoginCredentials(USERNAME, PASSWORD);

                    connProps.setLoginCredentials(loginCredentials);
                    connProps.setServerName(HOST);
                    connProps.setPortNumber(PORT);

                    if (app.getRegistrationStatus() != RegistrationStatus.REGISTERED)
                    {
                        app.registerApplication(TIMEOUT);
                    }
                    else
                    {
                        app.startConnection(TIMEOUT);
                    }

                    if (!SUP101DB.isSynchronized("default"))
                    {
                        SUP101DB.disableChangeLog();
                        SUP101DB.synchronize(); // Initial Synchronize

                        SynchronizationGroup sg = SUP101DB.getSynchronizationGroup("default");
                        sg.setEnableSIS(true);
                        sg.save();
                        SUP101DB.synchronize();
                    }
                    SUP101DB.enableChangeLog();
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
                finally
                {
                    initializationDone = true;
                }

                SUP101SampleActivity.this.runOnUiThread(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        adapter = new CustomerListAdapter(SUP101SampleActivity.this);

                        ListView listView = (ListView) findViewById(R.id.listView1);
                        listView.setAdapter(adapter);

                        listView.setOnItemClickListener(new OnItemClickListener()
                        {
                            @Override
                            public void onItemClick(AdapterView<?> a, View v, int position, long id)
                            {
                                Intent intent = new Intent(SUP101SampleActivity.this, DetailActivity.class);
                                intent.putExtra("sk", adapter.getSK(position));
                                SUP101SampleActivity.this.startActivityForResult(intent, REQUEST_DETAIL);
                            }

                        });
                    }

                });
            }
        }).start();

    }

    private class CustomerDBCallback extends DefaultCallbackHandler
    {
        @Override
        public int onSynchronize(GenericList<SynchronizationGroup> groups, SynchronizationContext context)
        {
            if (context.getStatus() == SynchronizationStatus.STARTING)
            {
                SUP101SampleActivity.this.runOnUiThread(new Runnable()
                {

                    @Override
                    public void run()
                    {
                        Toast.makeText(SUP101SampleActivity.this, "Synchronizing ... ", Toast.LENGTH_SHORT);
                    }
                });
            }
            else if (context.getStatus() == SynchronizationStatus.FINISHING || context.getStatus() == SynchronizationStatus.ASYNC_REPLAY_UPLOADED)
            {
                SUP101SampleActivity.this.runOnUiThread(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        Toast.makeText(SUP101SampleActivity.this, "Synchronize done", Toast.LENGTH_SHORT);
                    }
                });
                if (SUP101SampleActivity.this.adapter != null)
                {
                    SUP101SampleActivity.this.adapter.refreshUI(false);
                }
            }

            return SynchronizationAction.CONTINUE;
        }
    }
}